-------------------------------------------------------------------------------
-- File name : minor_modules.vhd
-- Title     : Minor modules
-- project   : ECE 485
-- Library   : multi-cycle processor
-- Author(s) : Ivan Castellanos
-- Purpose   : definition of multi-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity half_adder is
  port (a     : in  std_logic;
        b     : in  std_logic;
        sum   : out std_logic;
        c_out : out std_logic);
end half_adder;

architecture behv of half_adder is
begin  
  sum <= a xor b;
  c_out <= a and b;  
end behv;


library ieee;
use ieee.std_logic_1164.all;

entity counter6 is
  port (addrin	: in  std_logic_vector(5 downto 0);
        addrout	: out std_logic_vector(5 downto 0));
end counter6;

architecture counter6 of counter6 is

  component half_adder 
    port (a     : in  std_logic;
          b     : in  std_logic;
          sum   : out std_logic;
          c_out : out std_logic);
  end component;

  signal cout : std_logic_vector (4 downto 0);
  signal one : std_logic;

begin
  
  one <= '1';
  ha0: half_adder port map (a => addrin(0), b => one, sum => addrout(0), c_out => cout(0));
  ha1: half_adder port map (a => addrin(1), b => cout(0), sum => addrout(1), c_out => cout(1));
  ha2: half_adder port map (a => addrin(2), b => cout(1), sum => addrout(2), c_out => cout(2));
  ha3: half_adder port map (a => addrin(3), b => cout(2), sum => addrout(3), c_out => cout(3));
  ha4: half_adder port map (a => addrin(4), b => cout(3), sum => addrout(4), c_out => cout(4));
  ha5: half_adder port map (a => addrin(5), b => cout(4), sum => addrout(5));

end counter6;


library ieee;
use ieee.std_logic_1164.all;

entity lowzextend is
  port (a : in  std_logic_vector(15 downto 0);
        y : out std_logic_vector(31 downto 0));
end lowzextend;

architecture equations of lowzextend is
begin
  y(31 downto 16) <= a(15 downto 0);
  y(15) <= '0';
  y(14) <= '0';
  y(13) <= '0';
  y(12) <= '0';
  y(11) <= '0';
  y(10) <= '0';
  y(9)  <= '0';
  y(8)  <= '0';
  y(7)  <= '0';
  y(6)  <= '0';
  y(5)  <= '0';
  y(4)  <= '0';
  y(3)  <= '0';
  y(2)  <= '0';
  y(1)  <= '0';
  y(0)  <= '0';
end equations;


library ieee;
use ieee.std_logic_1164.all;

entity signextend is
  port (a : in  std_logic_vector(15 downto 0);
        y : out std_logic_vector(31 downto 0));
end signextend;

architecture equations of signextend is
begin
  y(15 downto 0) <= a(15 downto 0);
  y(16) <= a(15);
  y(17) <= a(15);
  y(18) <= a(15);
  y(19) <= a(15);
  y(20) <= a(15);
  y(21) <= a(15);
  y(22) <= a(15);
  y(23) <= a(15);
  y(24) <= a(15);
  y(25) <= a(15);
  y(26) <= a(15);
  y(27) <= a(15);
  y(28) <= a(15);
  y(29) <= a(15);
  y(30) <= a(15);
  y(31) <= a(15);
end equations;


library ieee;
use ieee.std_logic_1164.all;

entity shiftleft2 is
  port (a : in  std_logic_vector(31 downto 0);
        y : out std_logic_vector(31 downto 0));
end shiftleft2;

architecture equations of shiftleft2 is
begin
  y(0) <= '0';
  y(1) <= '0';
  y(2) <= a(0);
  y(3) <= a(1);
  y(4) <= a(2);
  y(5) <= a(3);
  y(6) <= a(4);
  y(7) <= a(5);
  y(8) <= a(6);
  y(9) <= a(7);
  y(10) <= a(8);
  y(11) <= a(9);
  y(12) <= a(10);
  y(13) <= a(11);
  y(14) <= a(12);
  y(15) <= a(13);
  y(16) <= a(14);
  y(17) <= a(15);
  y(18) <= a(16);
  y(19) <= a(17);
  y(20) <= a(18);
  y(21) <= a(19);
  y(22) <= a(20);
  y(23) <= a(21);
  y(24) <= a(22);
  y(25) <= a(23);
  y(26) <= a(24);
  y(27) <= a(25);
  y(28) <= a(26);
  y(29) <= a(27);
  y(30) <= a(28);
  y(31) <= a(29);
end equations;


library ieee;
use ieee.std_logic_1164.all;

entity bit_alu is
  port (a     : in  std_logic;
        b     : in  std_logic;
        c_in  : in  std_logic;
        less  : in  std_logic;
        op    : in  std_logic_vector(2 downto 0);
        x     : out std_logic;
        c_out : out std_logic);
end bit_alu;

architecture bit_alu of bit_alu is

  component mux41 
    port (a   : in  std_logic;
          b   : in  std_logic;
          c   : in  std_logic;
          d   : in  std_logic;
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic);
  end component;

  component mux21
    port (a   : in  std_logic;
          b   : in  std_logic;
          sel : in  std_logic;
          z   : out std_logic);
  end component;

  signal muxin : std_logic_vector (2 downto 0);
  signal binvert, bxored : std_logic;
  signal node_and, node_or, node_xor, node1, node2: std_logic;

begin
  binvert <= (op(2) and op(1)) ;					
  bxored <= (b xor binvert);						
  node_and <= (a and bxored);						
  node_or <= (a or bxored);						
  node_xor <= ((not node_and) and node_or);		
  
  mx1: mux21 port map (node_and, node_xor, op(2), muxin(0));	
  muxin(1) <= (op(2) xor node_or);							
  
  node1 <= (c_in and node_xor);
  node2 <= (node_xor or c_in);
  muxin(2) <= ((not node1) and node2);			
  
  c_out <= (node_and or node1);
  
  mx2: mux41 port map (muxin(0), muxin(1), muxin(2), less, op(1 downto 0), x);
  
end bit_alu;


library ieee;
use ieee.std_logic_1164.all;

entity bit_alu_msb is
  port (a        : in  std_logic;
        b        : in  std_logic;
        c_in     : in  std_logic;
        less     : in  std_logic;
        op       : in  std_logic_vector (2 downto 0);
        x        : out std_logic;
        set      : out std_logic;
        overflow : out std_logic);
end bit_alu_msb;

architecture bit_alu_msb of bit_alu_msb is

    component mux41 
    port (a   : in  std_logic;
          b   : in  std_logic;
          c   : in  std_logic;
          d   : in  std_logic;
          sel : in  std_logic_vector(1 downto 0);
          z   : out std_logic);
  end component;

  component mux21
    port (a   : in  std_logic;
          b   : in  std_logic;
          sel : in  std_logic;
          z   : out std_logic);
  end component;

    signal muxin : std_logic_vector (2 downto 0);
    signal binvert, bxored, c_out, z : std_logic;
    signal node_and, node_or, node_xor, node1, node2 : std_logic;

begin
  binvert <= (op(2) and op(1)) ;					
  bxored <= (b xor binvert);						
  node_and <= (a and bxored);						
  node_or <= (a or bxored);						
  node_xor <= ((not node_and) and node_or);		
  
  mx1: mux21 port map (node_and, node_xor, op(2), muxin(0));	
  muxin(1) <= (op(2) xor node_or);							
  
  node1 <= (c_in and node_xor);
  node2 <= (node_xor or c_in);
  muxin(2) <= ((not node1) and node2);			

  mx2: mux41 port map (muxin(0), muxin(1), muxin(2), less, op(1 downto 0), x);
  c_out <= (node_and or node1);
  z <= (c_in xor c_out);
  set <= muxin(2);								
  overflow <= (z and (op(1) and not op(0)));
end bit_alu_msb;



