-- 32 Bit ALU (Based on CLA)
-- Author: John Moskal
--  Created: 
-- Modified: 
--
-- Modifications:
--
------------------------------------------------------------------------------------------
-- 000 OR
-- 001 NOR
-- 010 AND
-- 011 XOR
-- 100 ADD
-- 101 SUB
-- 110 invalid
-- 111 SLT

library work;
library ieee;
use ieee.std_logic_1164.all;

entity ALU1CLA is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   SLT: in std_logic;
   OP:  in std_logic_vector(2 downto 0);
   p,g: out std_logic;
   R:   out std_logic
);
end entity ALU1CLA;

architecture stuff of ALU1CLA is

component RFAx is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   p,g: out std_logic;
   R:   out std_logic;
   X:   out std_logic
);
end component RFAx;

component MUX2 is
port
(
        A, B:           in std_logic;
        SL:             in std_logic;
        R:              out std_logic
);
end component MUX2;

component MUX4 is
port
(
        A, B, C, D:     in std_logic;
        SL:             in std_logic_vector(1 downto 0);
        R:              out std_logic
);
end component MUX4;

---
signal bx_w:      std_logic;
signal p_w:      std_logic;
signal np_w:     std_logic;
signal g_w:      std_logic;
signal addsub_w: std_logic;
signal x_w:      std_logic;
signal ornor_w:  std_logic;
signal andxor_w: std_logic;


begin

np_w <= not p_w;

bx_w <= B xor (OP(0) and OP(2));

RFAxdd:   RFAx port map(A, bx_w, cin, p_w, g_w, addsub_w, x_w);
mx2a:    mux2 port map(p_w, np_w, OP(0), ornor_w);
mx2b:    mux2 port map(g_w, x_w, OP(0), andxor_w);
mx4:     mux4 port map(ornor_w, andxor_w, addsub_w, slt, OP(2 downto 1), R);

p <= p_w;
g <= g_w;

end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity ALU1SCLA is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   SLT: in std_logic;
   OP:  in std_logic_vector(2 downto 0);
   p,g: out std_logic;
   R:   out std_logic;
   ADDSUB: out std_logic
);
end entity ALU1SCLA;

architecture stuff of ALU1SCLA is

component RFAx is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   p,g: out std_logic;
   R:   out std_logic;
   X:   out std_logic
);
end component RFAx;

component MUX2 is
port
(
        A, B:           in std_logic;
        SL:             in std_logic;
        R:              out std_logic
);
end component MUX2;

component MUX4 is
port
(
        A, B, C, D:     in std_logic;
        SL:             in std_logic_vector(1 downto 0);
        R:              out std_logic
);
end component MUX4;

---
signal bx_w:      std_logic;
signal p_w:      std_logic;
signal np_w:     std_logic;
signal g_w:      std_logic;
signal addsub_w: std_logic;
signal x_w:      std_logic;
signal ornor_w:  std_logic;
signal andxor_w: std_logic;


begin

np_w <= not p_w;

bx_w <= B xor (OP(0) and OP(2));

RFAxdd:   RFAx port map(A, bx_w, cin, p_w, g_w, addsub_w, x_w);
mx2a:    mux2 port map(p_w, np_w, OP(0), ornor_w);
mx2b:    mux2 port map(g_w, x_w, OP(0), andxor_w);
mx4:     mux4 port map(ornor_w, andxor_w, addsub_w, slt, OP(2 downto 1), R);

p <= p_w;
g <= g_w;

addsub <= addsub_w;

end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity ALU32CLA is
port
(
   A,B:             in std_logic_vector(31 downto 0);
   OP:              in std_logic_vector(2 downto 0);
   Result:          out std_logic_vector(31 downto 0);
   cout:            out std_logic;
   ovf:             out std_logic;
   zero:            out std_logic
);
end entity ALU32CLA;

architecture stuff of ALU32CLA is

component ALU1CLA is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   SLT: in std_logic;
   OP:  in std_logic_vector(2 downto 0);
   p,g: out std_logic;
   R:   out std_logic
);
end component ALU1CLA;

component ALU1SCLA is
port
(
   A,B: in std_logic;
   cin: in std_logic;
   SLT: in std_logic;
   OP:  in std_logic_vector(2 downto 0);
   p,g: out std_logic;
   R:   out std_logic;
   ADDSUB: out std_logic
);
end component ALU1SCLA;

component BCLG is
port
(
   pin:   in std_logic_vector(3 downto 0);
   gin:   in std_logic_vector(3 downto 0);
   c0:            in std_logic;
   c1,c2,c3:      out std_logic;
   P,G:           out std_logic
);
end component BCLG;

component BCLGL is
port
(
   p0,p1:   in std_logic;
   g0,g1:   in std_logic;
   c0:      in std_logic;
   c1:      out std_logic;
   cout:    out std_logic
);
end component BCLGL;

signal SLT:   std_logic_vector(31 downto 0);

signal cin_w: std_logic_vector(31 downto 0);
signal p_w, g_w: std_logic_vector(31 downto 0);
signal p2_w, g2_w: std_logic_vector(7 downto 0);
signal p3_w, g3_w: std_logic_vector(1 downto 0);
signal addsub_w: std_logic;
signal cout_w:   std_logic;
signal ovf_w: std_logic;
signal r_w: std_logic_vector(31 downto 0);

begin

cin_w(00) <= OP(0) and OP(2);

SLT(31 downto 1) <= "0000000000000000000000000000000";
    
RA00:   ALU1CLA port map(A(00), B(00), cin_w(00), SLT(00), OP, p_w(00), g_w(00), r_w(00));
RA01:   ALU1CLA port map(A(01), B(01), cin_w(01), SLT(01), OP, p_w(01), g_w(01), r_w(01));
RA02:   ALU1CLA port map(A(02), B(02), cin_w(02), SLT(02), OP, p_w(02), g_w(02), r_w(02));
RA03:   ALU1CLA port map(A(03), B(03), cin_w(03), SLT(03), OP, p_w(03), g_w(03), r_w(03));

RA04:   ALU1CLA port map(A(04), B(04), cin_w(04), SLT(04), OP, p_w(04), g_w(04), r_w(04));
RA05:   ALU1CLA port map(A(05), B(05), cin_w(05), SLT(05), OP, p_w(05), g_w(05), r_w(05));
RA06:   ALU1CLA port map(A(06), B(06), cin_w(06), SLT(06), OP, p_w(06), g_w(06), r_w(06));
RA07:   ALU1CLA port map(A(07), B(07), cin_w(07), SLT(07), OP, p_w(07), g_w(07), r_w(07));

RA08:   ALU1CLA port map(A(08), B(08), cin_w(08), SLT(08), OP, p_w(08), g_w(08), r_w(08));
RA09:   ALU1CLA port map(A(09), B(09), cin_w(09), SLT(09), OP, p_w(09), g_w(09), r_w(09));
RA10:   ALU1CLA port map(A(10), B(10), cin_w(10), SLT(10), OP, p_w(10), g_w(10), r_w(10));
RA11:   ALU1CLA port map(A(11), B(11), cin_w(11), SLT(11), OP, p_w(11), g_w(11), r_w(11));

RA12:   ALU1CLA port map(A(12), B(12), cin_w(12), SLT(12), OP, p_w(12), g_w(12), r_w(12));
RA13:   ALU1CLA port map(A(13), B(13), cin_w(13), SLT(13), OP, p_w(13), g_w(13), r_w(13));
RA14:   ALU1CLA port map(A(14), B(14), cin_w(14), SLT(14), OP, p_w(14), g_w(14), r_w(14));
RA15:   ALU1CLA port map(A(15), B(15), cin_w(15), SLT(15), OP, p_w(15), g_w(15), r_w(15));

RA16:   ALU1CLA port map(A(16), B(16), cin_w(16), SLT(16), OP, p_w(16), g_w(16), r_w(16));
RA17:   ALU1CLA port map(A(17), B(17), cin_w(17), SLT(17), OP, p_w(17), g_w(17), r_w(17));
RA18:   ALU1CLA port map(A(18), B(18), cin_w(18), SLT(18), OP, p_w(18), g_w(18), r_w(18));
RA19:   ALU1CLA port map(A(19), B(19), cin_w(19), SLT(19), OP, p_w(19), g_w(19), r_w(19));

RA20:   ALU1CLA port map(A(20), B(20), cin_w(20), SLT(20), OP, p_w(20), g_w(20), r_w(20));
RA21:   ALU1CLA port map(A(21), B(21), cin_w(21), SLT(21), OP, p_w(21), g_w(21), r_w(21));
RA22:   ALU1CLA port map(A(22), B(22), cin_w(22), SLT(22), OP, p_w(22), g_w(22), r_w(22));
RA23:   ALU1CLA port map(A(23), B(23), cin_w(23), SLT(23), OP, p_w(23), g_w(23), r_w(23));

RA24:   ALU1CLA port map(A(24), B(24), cin_w(24), SLT(24), OP, p_w(24), g_w(24), r_w(24));
RA25:   ALU1CLA port map(A(25), B(25), cin_w(25), SLT(25), OP, p_w(25), g_w(25), r_w(25));
RA26:   ALU1CLA port map(A(26), B(26), cin_w(26), SLT(26), OP, p_w(26), g_w(26), r_w(26));
RA27:   ALU1CLA port map(A(27), B(27), cin_w(27), SLT(27), OP, p_w(27), g_w(27), r_w(27));

RA28:   ALU1CLA port map(A(28), B(28), cin_w(28), SLT(28), OP, p_w(28), g_w(28), r_w(28));
RA29:   ALU1CLA port map(A(29), B(29), cin_w(29), SLT(29), OP, p_w(29), g_w(29), r_w(29));
RA30:   ALU1CLA port map(A(30), B(30), cin_w(30), SLT(30), OP, p_w(30), g_w(30), r_w(30));
RA31:   ALU1SCLA port map(A(31), B(31), cin_w(31), SLT(31), OP, p_w(31), g_w(31), r_w(31), addsub_w);

L100:   BCLG port map(p_w(03 downto 00), g_w(03 downto 00), cin_w(00), cin_w(01), cin_w(02), cin_w(03), p2_w(0), g2_w(0));
L101:   BCLG port map(p_w(07 downto 04), g_w(07 downto 04), cin_w(04), cin_w(05), cin_w(06), cin_w(07), p2_w(1), g2_w(1));
L102:   BCLG port map(p_w(11 downto 08), g_w(11 downto 08), cin_w(08), cin_w(09), cin_w(10), cin_w(11), p2_w(2), g2_w(2));
L103:   BCLG port map(p_w(15 downto 12), g_w(15 downto 12), cin_w(12), cin_w(13), cin_w(14), cin_w(15), p2_w(3), g2_w(3));
L104:   BCLG port map(p_w(19 downto 16), g_w(19 downto 16), cin_w(16), cin_w(17), cin_w(18), cin_w(19), p2_w(4), g2_w(4));
L105:   BCLG port map(p_w(23 downto 20), g_w(23 downto 20), cin_w(20), cin_w(21), cin_w(22), cin_w(23), p2_w(5), g2_w(5));
L106:   BCLG port map(p_w(27 downto 24), g_w(27 downto 24), cin_w(24), cin_w(25), cin_w(26), cin_w(27), p2_w(6), g2_w(6));
L107:   BCLG port map(p_w(31 downto 28), g_w(31 downto 28), cin_w(28), cin_w(29), cin_w(30), cin_w(31), p2_w(7), g2_w(7));

L200:   BCLG port map(p2_w(03 downto 0),  g2_w(3 downto 0), cin_w(00), cin_w(04), cin_w(08), cin_w(12), p3_w(0), g3_w(0));
L201:   BCLG port map(p2_w(07 downto 4),  g2_w(7 downto 4), cin_w(16), cin_w(20), cin_w(24), cin_w(28), p3_w(1), g3_w(1));

L3:    BCLGL port map(p3_w(0), p3_w(1), g3_w(0), g3_w(1), cin_w(00), cin_w(16), cout_w);

ovf_w <= cout_w xor cin_w(31);
SLT(0) <= ovf_w xor addsub_w;

cout <= cout_w;
ovf <= ovf_w;
zero <= not(r_w(31) or r_w(30) or r_w(29) or r_w(28) or r_w(27) or
            r_w(26) or r_w(25) or r_w(24) or r_w(23) or r_w(22) or
            r_w(21) or r_w(20) or r_w(19) or r_w(18) or r_w(17) or
            r_w(16) or r_w(15) or r_w(14) or r_w(13) or r_w(12) or
            r_w(11) or r_w(10) or r_w(09) or r_w(08) or r_w(07) or
            r_w(06) or r_w(05) or r_w(04) or r_w(03) or r_w(02) or
            r_w(01) or r_w(00));

Result <= r_w;

end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------


