-- Adaptive Predictor
-- Author: John Moskal
--  Created: 11-26-04
-- Modified: 11-26-04
--
-- Modifications:
--
------------------------------------------------------------------------------------------
-----------------------------------------------------------------------

library WORK;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
-------------------------------------------

entity shiftreg4 is
port
(
        input:  in std_logic;
        stb:    in std_logic;
        rst:    in std_logic;
        addr:   out std_logic_vector(3 downto 0)
);
end entity shiftreg4;
--------------------------------------------

architecture stuff of shiftreg4 is

component dff
port
(   
        D:      in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        Q:      out std_logic
);
end component;

signal  ad0_w:  std_logic;
signal  ad1_w:  std_logic;
signal  ad2_w:  std_logic;
signal  ad3_w:  std_logic;
signal  stb_w:  std_logic;
signal  rst_w:  std_logic;

begin

stb_w <= stb;
rst_w <= rst;

R0:     dff port map(input, stb_w, rst_w, ad0_w);
R1:     dff port map(ad0_w, stb_w, rst_w, ad1_w);
R2:     dff port map(ad1_w, stb_w, rst_w, ad2_w);
R3:     dff port map(ad2_w, stb_w, rst_w, ad3_w);

addr(0) <= ad0_w;
addr(1) <= ad1_w;
addr(2) <= ad2_w;
addr(3) <= ad3_w;

end stuff;

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------

library WORK;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
-------------------------------------------

entity predasm is
port
(
        enable: in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        R:      in std_logic;
        P:      out std_logic
);
end entity predasm;
-------------------------------------------

architecture stuff of predasm is

component dff
port
(
        D:      in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        Q:      out std_logic);
end component;

signal B0o_w:   std_logic;
signal B1o_w:   std_logic;

signal B0i_w:   std_logic;
signal B1i_w:   std_logic;

signal clk_w:   std_logic;
signal rst_w:   std_logic;

begin

rst_w <= rst;
clk_w <= (enable and clk);

--B0i_w <= (not B1o_w and not B0o_w and     R) or (B1o_w and     B0o_w) or (B1o_w and R);
--B1i_w <= (not B1o_w and     B0o_w and not R) or (B1o_w and not B0o_w) or (B1o_w and R);

B1i_w <= (    B0o_w and R) or (B1o_w and     B0o_w) or (B1o_w and R);
B0i_w <= (not B0o_w and R) or (B1o_w and not B0o_w) or (B1o_w and R);

B0:     dff port map(B0i_w, clk_w, rst_w, B0o_w);
B1:     dff port map(B1i_w, clk_w, rst_w, B1o_w);

P <= B1o_w;

end stuff;

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------

library WORK;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
-------------------------------------------
 
entity addrdec4s is 
port
(
        a:      in std_logic_vector(3 downto 0);
        s:      out std_logic_vector(15 downto 0)
); 
end entity addrdec4s; 
-------------------------------------------
 
architecture stuff of addrdec4s is 
begin 
        s(0) <=  (not a(3) and not a(2) and not a(1) and not a(0));
        s(1) <=  (not a(3) and not a(2) and not a(1) and     a(0));
        s(2) <=  (not a(3) and not a(2) and     a(1) and not a(0));
        s(3) <=  (not a(3) and not a(2) and     a(1) and     a(0));
        s(4) <=  (not a(3) and     a(2) and not a(1) and not a(0));
        s(5) <=  (not a(3) and     a(2) and not a(1) and     a(0));
        s(6) <=  (not a(3) and     a(2) and     a(1) and not a(0));
        s(7) <=  (not a(3) and     a(2) and     a(1) and     a(0));
        s(8) <=  (    a(3) and not a(2) and not a(1) and not a(0));
        s(9) <=  (    a(3) and not a(2) and not a(1) and     a(0));
        s(10) <= (    a(3) and not a(2) and     a(1) and not a(0));
        s(11) <= (    a(3) and not a(2) and     a(1) and     a(0));
        s(12) <= (    a(3) and     a(2) and not a(1) and not a(0));
        s(13) <= (    a(3) and     a(2) and not a(1) and     a(0));
        s(14) <= (    a(3) and     a(2) and     a(1) and not a(0));
        s(15) <= (    a(3) and     a(2) and     a(1) and     a(0));
end stuff; 

-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------
-----------------------------------------------------------------------


library WORK;
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
-------------------------------------------

entity predictor is 
port
(
        stb:    in std_logic;
        rst:    in std_logic;
        R:      in std_logic;
        P:      out std_logic
); 
end entity predictor; 
-------------------------------------------

architecture stuff of predictor is

component reg4 is
port
(
        input:  in std_logic_vector(3 downto 0);
        stb:    in std_logic;
        rst:    in std_logic;
        output:   out std_logic_vector(3 downto 0)
);
end component;

component shiftreg4
port
(
        input:  in std_logic;
        stb:    in std_logic;
        rst:    in std_logic;
        addr:   out std_logic_vector(3 downto 0)
);
end component;

component predasm
port
(
        enable: in std_logic;
        clk:    in std_logic;
        rst:    in std_logic;
        R:      in std_logic;
        P:      out std_logic
);
end component;

component mux16to1
port
(
        sel:    in std_logic_vector(3 downto 0); 
        inp:    in std_logic_vector(15 downto 0);  
        outp:   out std_logic
);
end component;

component addrdec4s
port
(
        a:      in std_logic_vector(3 downto 0);
        s:      out std_logic_vector(15 downto 0)
);
end component;

-----

signal addr_w:          std_logic_vector(3 downto 0);
signal addrbuf_w:       std_logic_vector(3 downto 0);
signal enab_w:          std_logic_vector(15 downto 0);
signal pred_w:          std_logic_vector(15 downto 0);
signal rslt_w:          std_logic;
signal strb_w:          std_logic;
signal nstrb_w:         std_logic;
signal rset_w:          std_logic;

-----

begin

rslt_w <= R;
strb_w <= stb;
nstrb_w <= not stb;
rset_w <= rst;

bht:    shiftreg4 port map(rslt_w, strb_w, rset_w, addr_w);
buf:    reg4 port map(addr_w, nstrb_w, rset_w, addrbuf_w);
dec:    addrdec4s port map(addrbuf_w, enab_w);

pm0:    predasm port map(enab_w(0),  strb_w, rset_w, rslt_w, pred_w(0));
pm1:    predasm port map(enab_w(1),  strb_w, rset_w, rslt_w, pred_w(1));
pm2:    predasm port map(enab_w(2),  strb_w, rset_w, rslt_w, pred_w(2));
pm3:    predasm port map(enab_w(3),  strb_w, rset_w, rslt_w, pred_w(3));
pm4:    predasm port map(enab_w(4),  strb_w, rset_w, rslt_w, pred_w(4));
pm5:    predasm port map(enab_w(5),  strb_w, rset_w, rslt_w, pred_w(5));
pm6:    predasm port map(enab_w(6),  strb_w, rset_w, rslt_w, pred_w(6));
pm7:    predasm port map(enab_w(7),  strb_w, rset_w, rslt_w, pred_w(7));
pm8:    predasm port map(enab_w(8),  strb_w, rset_w, rslt_w, pred_w(8));
pm9:    predasm port map(enab_w(9),  strb_w, rset_w, rslt_w, pred_w(9));
pma:    predasm port map(enab_w(10), strb_w, rset_w, rslt_w, pred_w(10));
pmb:    predasm port map(enab_w(11), strb_w, rset_w, rslt_w, pred_w(11));
pmc:    predasm port map(enab_w(12), strb_w, rset_w, rslt_w, pred_w(12));
pmd:    predasm port map(enab_w(13), strb_w, rset_w, rslt_w, pred_w(13));
pme:    predasm port map(enab_w(14), strb_w, rset_w, rslt_w, pred_w(14));
pmf:    predasm port map(enab_w(15), strb_w, rset_w, rslt_w, pred_w(15));

mux:    mux16to1 port map(addr_w, pred_w, P);

end stuff;
